package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;

import org.apache.log4j.Logger;

import eu.clarin.cmdi.mdservice.action.GenericAction;
import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Termset;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 * Base class for implementation of proxies to target systems.
 * This is now (after rework 2011-09) decoupled from the called Action. 
 * 
 * Defines the methods used for retrieving the data 
 * and provides basic implementation where possible.
 *
 * In contrast to base class BasicProxy RepoProxy tries to get the base_uri from WorkspaceProfile (based on the repository-parameter)
 *  
 * @author vronk
 *
 */
public class DCRProxy extends BasicProxy{

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("DCRProxy");
 
	private String proxy_key = "dcr";
		
	private GenericAction source_action;
	
	public GenericAction getSourceAction() {
			return source_action;
	}
	
	@Override
	public void setSourceAction(GenericAction action) {		
		source_action = action;
	}
	
	public void checkParams() {
		// required Params
		// check and set defaults
//		if (WorkspaceProfile.getRepositoryPath(getParam("repository")) == null) {
	//		getSourceAction().getDiagnostics().Add(Diagnostic.UNSUPPORTED_PARAMETERVALUE, "repository");
		//}		
	}
	
	/**
	 * identification of the target-proxy (type)
	 * base for finding the right base_url in the props
	 * subclass has to override with its specific proxykey
	 * @return
	 */
	public String getProxyKey() {				
			return proxy_key;		
	}

	/**
	 * Override of the basic method provided by the super-class:
	 * Read the repository-uri from the WorkspaceProfile based on the repository-parameter 
	 */
		
	public String getBaseURI() {		
		//String uri = WorkspaceProfile.getTermsetProperty("isocat", "url");		
		String uri = getDCRAttr("url");
		return uri;
	}

	/**
	 * Provides the base-URL specific to given target repository or registry 
	 * @return
	 * @throws MalformedURLException
	 */	
	public URL getBaseURL() throws MalformedURLException {		
		URL base_url = new URL(getBaseURI());		
		return base_url;
	}
	
	/** all getDCR* functions taken from DCRProxyAction - may need check
	 * Getting the appropriate Termset 
	 * @return
	 */
	public Termset getDCR() {		
		String dcr_id="";
		if (getParam("q")!=null && !getParam("q").equals("")) dcr_id = getParam("q");
		// override with dcrid-param

	// if (getDcrid()!=null && !getDcrid().equals("")) dcr_id = getDcrid();			

		Termset res = (Termset) Termset.getTermset(dcr_id);
		if (res==null) log.debug("getDCR(): no termset found for:" + dcr_id);
		//else
			//log.debug("getDCR():" + res.getAttr("id") + ":" + res.getAttr("url") );
		
		return res;
	}
	
	public String getDCRAttr(String key) {	
		Termset dcr = getDCR();
		String val=""; 
		if (dcr!=null) val= dcr.getAttr(key);
		 return val;
	}

	
	public InputStream getSourceStream() throws IOException, NoStylesheetException, CQLParseException {
		
		URLConnection urlConnection = getTargetRequest().openConnection();
		
		if (getDCRAttr("format").equals("dcif")) {		
			//urlConnection.setRequestProperty("Accept", "application/dcif+xml");
			urlConnection.setRequestProperty("Accept", "application/rdf+xml");			
			log.debug("setting header Accept:" + urlConnection.getRequestProperty("Accept"));
		}
        //urlConnection.setRequestProperty("Accept-Language", getLang());
        
	 	InputStream resultStream = urlConnection.getInputStream();
		
        return resultStream;    
	}

	
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */	
	public URL getTargetRequest() throws IOException, CQLParseException {
		//URL targetURL =new URL( getBaseURL(), urls.get(getActionkey()) + query.toURLParam() );
		URL targetURL =getBaseURL();
		
		log.debug("getTargetRequest().targetURL:" + targetURL);
	    return targetURL;
		
	}

	

}	